/*
A demo porgram that opens a GPX file and parses the longitud and latitud data for the waypoints.
Axel Brinkeby oktober 2012
*/
/**********************************************************************************************/
// PREPROCESSORS
#include <iostream>                                                             // cin, cout
#include <fstream>                                                              // for reading files
#include <iomanip>                                                              // for formated output
#include <string>                                                               // for using strings
#include <cstring>                                                              // for converiting strings to c-strings
#include <cstdlib>                                                              // converting a C-string to a double, etc.

using namespace std;
/**********************************************************************************************/
// CONSTANTS
const int MAX_LINES = 200;                                                      // Maxumum amount of lines in the GPX file
const int MAX_WAYPOINTS = 100;                                                  // Maximum number of waypoints
const int MENU_W = 79; 					                                        // the with of the menu-lines
const char menu_fill = '-';                                                     // the char builing up the menu

struct Waypoint                                                                 // a waypoint
{
    double lat;                                                                 // latitude of the waypoint
    double lon;                                                                 // longitude of the waypoint
};

/**********************************************************************************************/
// FUNCTION PROTOTYPES

// let the user open a GPX file with waypoint-data, and save the data to the array, return number of waypoint red into the file
int getWaypoints(Waypoint wp[]);

// display a list of all n waypoints
void putWaypoints(Waypoint wp[], int n);

/**********************************************************************************************/


int main()
{
    Waypoint waypoints[MAX_WAYPOINTS];

    int number_of_waypoints = getWaypoints(waypoints);                          // open a GPX file, save the data to the array
    putWaypoints(waypoints, number_of_waypoints);                               // display all waypoints

    return(0);
}

/**********************************************************************************************/
// FUNCTIONS

int getWaypoints(Waypoint wp[]) {
    bool file_is_open = false;                                                  // set to true when the file opens, else, it is used to let the user try to open the file again
    string filename = "x.txt";                                                  // filename of the GPX file to open
    string lines[MAX_LINES];                                                    // the lines red from the file
    int line_index = 0;                                                         // used while reading the file
    int wp_index = 0;                                                          // used to display the waypoints
    size_t lat_pointer;                                                         // used for parsing the lat and lon data from the GPX file
    size_t lon_pointer;

    cout << "Enter name (including extention) of GPX file to open: ";
    cin >> filename;

    reload_file:        // this label is used insted of a do-while-loop to make the filestream accesible in the entire main function
        ifstream GPXfile(filename.c_str());                                     // open the text (GPX) file

        if (!GPXfile.is_open()) {                                               // if the file failed to open, display error, and ask for a new filename
            cout << endl << "The file \"" << filename << "\" was not found, or could not be opened. " << endl << "Try again: ";
            cin >> filename;
        } else
            file_is_open = true;

    if (!file_is_open) goto reload_file;                                        // try to open the file again with a new filename if it faild the first time

    getline(GPXfile, lines[line_index]);                                        // read the first line of the file

    while (!GPXfile.eof()) {                                                    // read untill the end of the file.
        line_index++;
        getline(GPXfile, lines[line_index]);
    }

    GPXfile.close();                                                            // close the file, it is not needed any more

    for (int i = 0; i < line_index; i++) {                                      // search trough all lines read from the file
        lat_pointer = lines[i].find("lat");
        lon_pointer = lines[i].find("lon");

        if (lat_pointer != string::npos) {                                      // if the latitude was found, we expect that the longitude also was found
            wp[wp_index].lat = atof(lines[i].substr(int(lat_pointer)+5, 8).c_str());
            wp[wp_index].lon = atof(lines[i].substr(int(lon_pointer)+5, 8).c_str());  // convert and save the lat and lon data as doubles in the wp-array
            wp_index++;
        }
    }

    wp[0].lat = 0;
    wp[0].lon = 0;
    return wp_index;
}


void putWaypoints(Waypoint wp[], int n) {
    cout << endl;                                                               // display the waypoints in a list
    cout << setfill(menu_fill) << setw(MENU_W) << "" << setfill(' ') << endl;   // display the table header
    cout << left << setw(10) << "WP Nr:" << setw(14) << "Latitude:" << setw(14) << "Longitude:" << endl;
    cout << setfill(menu_fill) << setw(MENU_W) << "" << setfill(' ') << endl;

    for (int i = 1; i < n; i++)                                                 // display the numbers in the tables
        cout << left << setw(10) << fixed << setprecision(5) << i << setw(14) << wp[i].lat << setw(14) << wp[i].lon << endl;
}
